class AppError extends Error {
  constructor (m) {
    super(m)
    this.code = 500
    this.status = this.constructor.name
  }

  toPlainObject () {
    return {
      id: this.id || this.code,
      status: this.status,
      code: this.code,
      title: this.title,
      detail: this.detail,
      meta: this.meta
    }
  }

  toString () {
    return JSON.stringify(this.toPlainObject())
  }
}
module.exports.AppError = AppError

class DatabaseError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Database internal error'
    this.detail = m
    this.code = 500
  }
}
module.exports.DatabaseError = DatabaseError

class NotFoundError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Resource not found'
    this.detail = m
    this.code = 404
  }
}
module.exports.NotFoundError = NotFoundError

class MissingArgumentError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Missing argument'
    this.detail = m
    this.code = 400
  }
}
module.exports.MissingArgumentError = MissingArgumentError

class ConflictError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Duplicated resource'
    this.code = 409
  }
}
module.exports.ConflictError = ConflictError

class IdMismatchError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Parameter from path do not match with body'
    this.detail = m
    this.code = 400
  }
}
module.exports.IdMismatchError = IdMismatchError

class UnauthorizedError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Not authorized'
    this.detail = m
    this.code = 401
  }
}
module.exports.ForbiddenError = UnauthorizedError

class ForbiddenError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Not allowed'
    this.detail = m
    this.code = 403
  }
}
module.exports.ForbiddenError = ForbiddenError

class MethodNotAllowedError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Requested method is not allowed'
    this.detail = m
    this.code = 405
  }
}
module.exports.ForbiddenError = MethodNotAllowedError

class NotImplementedError extends AppError {
  constructor (m) {
    super(m)
    this.title = 'Request not implemented'
    this.detail = m
    this.code = 501
  }
}
module.exports.NotImplementedError = NotImplementedError

class UnsupportedMediaTypeError extends AppError {
  constructor (m) {
    super(m)
    this.title =
      "Content-Type: application/vnd.api+json doesn't accept media type parameters"
    this.detail = m
    this.code = 415
  }
}
module.exports.UnsupportedMediaTypeError = UnsupportedMediaTypeError

class NotAcceptableTypeError extends AppError {
  constructor (m) {
    super(m)
    this.detail = m
    this.code = 406
  }
}
module.exports.NotAcceptableTypeError = NotAcceptableTypeError
